/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.modules.qpool.model;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.GenericGenerator;
import org.olat.core.id.CreateInfo;
import org.olat.core.id.ModifiedInfo;
import org.olat.core.id.Persistable;
import org.olat.modules.qpool.TaxonomyLevel;

/**
 * 
 * Initial date: 20.02.2013<br>
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
@Entity(name="qtaxonomylevel")
@Table(name="o_qp_taxonomy_level")
@NamedQueries({
	@NamedQuery(name="loadTaxonomyLevelByKey", query="select f from qtaxonomylevel f where f.key=:key"),
	@NamedQuery(name="loadTaxonomicPath", query="select f from qtaxonomylevel f where f.parentField.key=:parentKey"),
	@NamedQuery(name="loadTaxonomyDescendants", query="select f from qtaxonomylevel f  where f.materializedPathKeys like :path"),
	@NamedQuery(name="loadAllTaxonomyLevels", query="select f from qtaxonomylevel f  left join fetch f.parentField pf")
})
public class TaxonomyLevelImpl implements TaxonomyLevel, CreateInfo, ModifiedInfo, Persistable  {

	private static final long serialVersionUID = -1150399691749897973L;

	@Id
  @GeneratedValue(generator = "system-uuid")
  @GenericGenerator(name = "system-uuid", strategy = "hilo")
	@Column(name="id", nullable=false, unique=true, insertable=true, updatable=false)
	private Long key;
	
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="creationdate", nullable=false, insertable=true, updatable=false)
	private Date creationDate;
	
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="lastmodified", nullable=false, insertable=true, updatable=true)
	private Date lastModified;
	
	@Column(name="q_field", nullable=false, insertable=true, updatable=true)
	private String field;
	
	@Column(name="q_mat_path_ids", nullable=false, insertable=true, updatable=true)
	private String materializedPathKeys;
	@Column(name="q_mat_path_names", nullable=false, insertable=true, updatable=true)
	private String materializedPathNames;
	
	@ManyToOne(targetEntity=TaxonomyLevelImpl.class)
  @JoinColumn(name="fk_parent_field", nullable=true, insertable=true, updatable=true)
	private TaxonomyLevel parentField;
	
	@Override
	public Long getKey() {
		return key;
	}
	
	public void setKey(Long key) {
		this.key = key;
	}

	@Override
	public Date getCreationDate() {
		return creationDate;
	}

	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
	}

	@Override
	public Date getLastModified() {
		return lastModified;
	}

	@Override
	public void setLastModified(Date date) {
		this.lastModified = date;
	}
	
	public String getField() {
		return field;
	}

	public void setField(String field) {
		this.field = field;
	}

	public TaxonomyLevel getParentField() {
		return parentField;
	}

	public void setParentField(TaxonomyLevel parentField) {
		this.parentField = parentField;
	}

	public String getMaterializedPathKeys() {
		return materializedPathKeys;
	}

	public void setMaterializedPathKeys(String materializedPathKeys) {
		this.materializedPathKeys = materializedPathKeys;
	}

	public String getMaterializedPathNames() {
		return materializedPathNames;
	}

	public void setMaterializedPathNames(String materializedPathNames) {
		this.materializedPathNames = materializedPathNames;
	}

	@Override
	public int hashCode() {
		return key == null ? 97489 : key.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if(this == obj) {
			return true;
		}
		if(obj instanceof TaxonomyLevelImpl) {
			TaxonomyLevelImpl f = (TaxonomyLevelImpl)obj;
			return key != null && key.equals(f.key);
		}
		return false;
	}

	@Override
	public boolean equalsByPersistableKey(Persistable persistable) {
		return equals(persistable);
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("studyField[key=").append(this.key)
			.append("]").append(super.toString());
		return sb.toString();
	}
}
